#!/bin/bash

if [ $# -lt 1 ];then
  echo "use the file name of the sequence file as first parameter"
  exit 1
fi

filename="$1"

# autoactivate on

cat <<EOF
#!plantuml
@startuml
scale 1

title <size:30>automatically generated sequence diagram</size>

participant main
EOF


declare -A colorArray
colorArray[0]="#000000"
colorArray[1]="#0000ff"
colorArray[2]="#ff0000"
colorArray[3]="#00ff00"
colorArray[4]="#00ffff"

declare -A objectSeen

for objectName in $(grep "^call:" "$filename" | sed -e 's|call: \([^\.]*\)\..*|\1|g' -e 's|::|@|g' | awk '!seen[$0]++');do
  aliasname=$(echo "$objectName" | sed -e 's|_.*||' -e 's|@|::|g')
  object_id=$(echo "$objectName" | sed -e 's|.*_\([0-9]*\)$|\1|')
  if [ "${objectSeen[$object_id]}" != "1" ];then
    objectSeen[$object_id]="1"
    echo participant \"$aliasname\" as $object_id
  fi
done

declare -A thread_color

let color_index=0
for thread_id in $(sed 's|.* \(thread_.*\)|\1|' "$filename" | awk '!seen[$0]++');do
  # echo "Color: ${colorArray[$color_index]} for thread: $thread_id"
  thread_color[$thread_id]=${colorArray[$color_index]}
  let color_index++
done

echo

declare -A classArray
declare -A currentClass

firstThread=$(head -n 1 "$filename" | sed 's|.* ||g')

classArray[$firstThread,0]="main"
currentClass[$firstThread]=0

cat "$filename" | while read line;do

  currentThread=$(echo "$line" | sed 's|.* \(thread_.*\)|\1|g')
  if [ "${line:0:7}" == "return:" ];then
    class=${classArray[$currentThread,${currentClass[$currentThread]}]}
    let currentClass[$currentThread]--
    # echo "$currentThread - ${currentClass[$currentThread]}"
    echo "$class -[${thread_color[$currentThread]}]-> ${classArray[$currentThread,${currentClass[$currentThread]}]}"

  elif [ "${line:0:5}" == "call:" ];then
    method=$(echo "$line" | sed 's|.*\.\([^ ]*\) .*|\1|g')
    class=$(echo "${line:6}" | sed -e 's|\..*||g' -e 's|^.*_||g')
    if [ "$method" == "initialize" ];then
      newObject="** "
    else
      newObject=""
    fi
    echo "${classArray[$currentThread,${currentClass[$currentThread]}]} [${thread_color[$currentThread]}]-> $class $newObject: $method"
    let currentClass[$currentThread]++
    # echo "$currentThread - ${currentClass[$currentThread]}"
    classArray[$currentThread,${currentClass[$currentThread]}]=$class

  elif [ "${line:0:11}" == "new_thread:" ];then
    class=$(echo "$line" | sed -e 's|^new_thread: \([^ ]*\) .*|\1|g' -e 's|^.*_||g')
    ismain=$(echo "$line" | sed 's|^new_thread: Object_[0-9]* thread_[0-9]*||g')
    if [ "$ismain" == "" ];then
      class="main"
    fi
    # echo " === new thread: $currentThread, $class"
    echo "$class [${thread_color[$currentThread]}]-> $class : newThread"
    classArray[$currentThread,0]="$class"
    currentClass[$currentThread]=0
  fi
done

echo
echo @enduml
